# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.accesslog.v3 import accesslog_pb2 as envoy_dot_config_dot_accesslog_dot_v3_dot_accesslog__pb2
from envoy.config.core.v3 import address_pb2 as envoy_dot_config_dot_core_dot_v3_dot_address__pb2
from envoy.config.core.v3 import base_pb2 as envoy_dot_config_dot_core_dot_v3_dot_base__pb2
from envoy.config.core.v3 import config_source_pb2 as envoy_dot_config_dot_core_dot_v3_dot_config__source__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from envoy.config.core.v3 import protocol_pb2 as envoy_dot_config_dot_core_dot_v3_dot_protocol__pb2
from envoy.config.core.v3 import substitution_format_string_pb2 as envoy_dot_config_dot_core_dot_v3_dot_substitution__format__string__pb2
from envoy.config.route.v3 import route_pb2 as envoy_dot_config_dot_route_dot_v3_dot_route__pb2
from envoy.config.route.v3 import scoped_route_pb2 as envoy_dot_config_dot_route_dot_v3_dot_scoped__route__pb2
from envoy.config.trace.v3 import http_tracer_pb2 as envoy_dot_config_dot_trace_dot_v3_dot_http__tracer__pb2
from envoy.type.http.v3 import path_transformation_pb2 as envoy_dot_type_dot_http_dot_v3_dot_path__transformation__pb2
from envoy.type.tracing.v3 import custom_tag_pb2 as envoy_dot_type_dot_tracing_dot_v3_dot_custom__tag__pb2
from envoy.type.v3 import percent_pb2 as envoy_dot_type_dot_v3_dot_percent__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from xds.type.matcher.v3 import matcher_pb2 as xds_dot_type_dot_matcher_dot_v3_dot_matcher__pb2
from envoy.annotations import deprecation_pb2 as envoy_dot_annotations_dot_deprecation__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import security_pb2 as udpa_dot_annotations_dot_security__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from udpa.annotations import versioning_pb2 as udpa_dot_annotations_dot_versioning__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nYenvoy/extensions/filters/network/http_connection_manager/v3/http_connection_manager.proto\x12;envoy.extensions.filters.network.http_connection_manager.v3\x1a)envoy/config/accesslog/v3/accesslog.proto\x1a\"envoy/config/core/v3/address.proto\x1a\x1f\x65nvoy/config/core/v3/base.proto\x1a(envoy/config/core/v3/config_source.proto\x1a$envoy/config/core/v3/extension.proto\x1a#envoy/config/core/v3/protocol.proto\x1a\x35\x65nvoy/config/core/v3/substitution_format_string.proto\x1a!envoy/config/route/v3/route.proto\x1a(envoy/config/route/v3/scoped_route.proto\x1a\'envoy/config/trace/v3/http_tracer.proto\x1a,envoy/type/http/v3/path_transformation.proto\x1a&envoy/type/tracing/v3/custom_tag.proto\x1a\x1b\x65nvoy/type/v3/percent.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a!xds/type/matcher/v3/matcher.proto\x1a#envoy/annotations/deprecation.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1fudpa/annotations/security.proto\x1a\x1dudpa/annotations/status.proto\x1a!udpa/annotations/versioning.proto\x1a\x17validate/validate.proto\"\x9c\x39\n\x15HttpConnectionManager\x12z\n\ncodec_type\x18\x01 \x01(\x0e\x32\\.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.CodecTypeB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x1c\n\x0bstat_prefix\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12O\n\x03rds\x18\x03 \x01(\x0b\x32@.envoy.extensions.filters.network.http_connection_manager.v3.RdsH\x00\x12\x41\n\x0croute_config\x18\x04 \x01(\x0b\x32).envoy.config.route.v3.RouteConfigurationH\x00\x12\x62\n\rscoped_routes\x18\x1f \x01(\x0b\x32I.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRoutesH\x00\x12]\n\x0chttp_filters\x18\x05 \x03(\x0b\x32G.envoy.extensions.filters.network.http_connection_manager.v3.HttpFilter\x12\x32\n\x0e\x61\x64\x64_user_agent\x18\x06 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12k\n\x07tracing\x18\x07 \x01(\x0b\x32Z.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.Tracing\x12X\n\x1c\x63ommon_http_protocol_options\x18# \x01(\x0b\x32).envoy.config.core.v3.HttpProtocolOptionsB\x07\x8a\x93\xb7*\x02\x08\x01\x12*\n\"http1_safe_max_connection_duration\x18: \x01(\x08\x12I\n\x15http_protocol_options\x18\x08 \x01(\x0b\x32*.envoy.config.core.v3.Http1ProtocolOptions\x12S\n\x16http2_protocol_options\x18\t \x01(\x0b\x32*.envoy.config.core.v3.Http2ProtocolOptionsB\x07\x8a\x93\xb7*\x02\x08\x01\x12J\n\x16http3_protocol_options\x18, \x01(\x0b\x32*.envoy.config.core.v3.Http3ProtocolOptions\x12 \n\x0bserver_name\x18\n \x01(\tB\x0b\xfa\x42\x08r\x06\xc0\x01\x02\xc8\x01\x00\x12\x9d\x01\n\x1cserver_header_transformation\x18\" \x01(\x0e\x32m.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.ServerHeaderTransformationB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12V\n\x1cscheme_header_transformation\x18\x30 \x01(\x0b\x32\x30.envoy.config.core.v3.SchemeHeaderTransformation\x12H\n\x16max_request_headers_kb\x18\x1d \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\n\xfa\x42\x07*\x05\x18\x80@ \x00\x12?\n\x13stream_idle_timeout\x18\x18 \x01(\x0b\x32\x19.google.protobuf.DurationB\x07\x8a\x93\xb7*\x02\x08\x01\x12\x37\n\x14stream_flush_timeout\x18; \x01(\x0b\x32\x19.google.protobuf.Duration\x12;\n\x0frequest_timeout\x18\x1c \x01(\x0b\x32\x19.google.protobuf.DurationB\x07\x8a\x93\xb7*\x02\x08\x01\x12K\n\x17request_headers_timeout\x18) \x01(\x0b\x32\x19.google.protobuf.DurationB\x0f\xfa\x42\x05\xaa\x01\x02\x32\x00\x8a\x93\xb7*\x02\x08\x01\x12\x30\n\rdrain_timeout\x18\x0c \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x38\n\x15\x64\x65layed_close_timeout\x18\x1a \x01(\x0b\x32\x19.google.protobuf.Duration\x12\x38\n\naccess_log\x18\r \x03(\x0b\x32$.envoy.config.accesslog.v3.AccessLog\x12U\n\x19\x61\x63\x63\x65ss_log_flush_interval\x18\x36 \x01(\x0b\x32\x19.google.protobuf.DurationB\x17\x18\x01\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x34\n\x1f\x66lush_access_log_on_new_request\x18\x37 \x01(\x08\x42\x0b\x18\x01\x92\xc7\x86\xd8\x04\x03\x33.0\x12\x82\x01\n\x12\x61\x63\x63\x65ss_log_options\x18\x38 \x01(\x0b\x32\x66.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.HcmAccessLogOptions\x12?\n\x12use_remote_address\x18\x0e \x01(\x0b\x32\x1a.google.protobuf.BoolValueB\x07\x8a\x93\xb7*\x02\x08\x01\x12\x1c\n\x14xff_num_trusted_hops\x18\x13 \x01(\r\x12T\n original_ip_detection_extensions\x18. \x03(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\x12T\n early_header_mutation_extensions\x18\x34 \x03(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\x12\x89\x01\n\x17internal_address_config\x18\x19 \x01(\x0b\x32h.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.InternalAddressConfig\x12\x17\n\x0fskip_xff_append\x18\x15 \x01(\x08\x12\x18\n\x03via\x18\x16 \x01(\tB\x0b\xfa\x42\x08r\x06\xc0\x01\x02\xc8\x01\x00\x12\x37\n\x13generate_request_id\x18\x0f \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12$\n\x1cpreserve_external_request_id\x18  \x01(\x08\x12)\n!always_set_request_id_in_response\x18% \x01(\x08\x12\x9a\x01\n\x1b\x66orward_client_cert_details\x18\x10 \x01(\x0e\x32k.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.ForwardClientCertDetailsB\x08\xfa\x42\x05\x82\x01\x02\x10\x01\x12\x97\x01\n\x1fset_current_client_cert_details\x18\x11 \x01(\x0b\x32n.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.SetCurrentClientCertDetails\x12\x41\n\x1b\x66orward_client_cert_matcher\x18< \x01(\x0b\x32\x1c.xds.type.matcher.v3.Matcher\x12\x1a\n\x12proxy_100_continue\x18\x12 \x01(\x08\x12\x39\n1represent_ipv4_remote_address_as_ipv4_mapped_ipv6\x18\x14 \x01(\x08\x12y\n\x0fupgrade_configs\x18\x17 \x03(\x0b\x32`.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.UpgradeConfig\x12\x32\n\x0enormalize_path\x18\x1e \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x15\n\rmerge_slashes\x18! \x01(\x08\x12\x99\x01\n path_with_escaped_slashes_action\x18- \x01(\x0e\x32o.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.PathWithEscapedSlashesAction\x12m\n\x14request_id_extension\x18$ \x01(\x0b\x32O.envoy.extensions.filters.network.http_connection_manager.v3.RequestIDExtension\x12i\n\x12local_reply_config\x18& \x01(\x0b\x32M.envoy.extensions.filters.network.http_connection_manager.v3.LocalReplyConfig\x12\x39\n\x18strip_matching_host_port\x18\' \x01(\x08\x42\x17\xf2\x98\xfe\x8f\x05\x11\x12\x0fstrip_port_mode\x12\x1d\n\x13strip_any_host_port\x18* \x01(\x08H\x01\x12H\n$stream_error_on_invalid_http_message\x18( \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x8f\x01\n\x1apath_normalization_options\x18+ \x01(\x0b\x32k.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.PathNormalizationOptions\x12\x1f\n\x17strip_trailing_host_dot\x18/ \x01(\x08\x12\x81\x01\n\x13proxy_status_config\x18\x31 \x01(\x0b\x32\x64.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.ProxyStatusConfig\x12R\n\x1etyped_header_validation_config\x18\x32 \x01(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\x12\x1f\n\x17\x61ppend_x_forwarded_port\x18\x33 \x01(\x08\x12\x1d\n\x15\x61ppend_local_overload\x18\x39 \x01(\x08\x12G\n#add_proxy_protocol_connection_state\x18\x35 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x1a\xfc\x04\n\x07Tracing\x12/\n\x0f\x63lient_sampling\x18\x03 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12/\n\x0frandom_sampling\x18\x04 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12\x30\n\x10overall_sampling\x18\x05 \x01(\x0b\x32\x16.envoy.type.v3.Percent\x12\x0f\n\x07verbose\x18\x06 \x01(\x08\x12\x39\n\x13max_path_tag_length\x18\x07 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x35\n\x0b\x63ustom_tags\x18\x08 \x03(\x0b\x32 .envoy.type.tracing.v3.CustomTag\x12\x35\n\x08provider\x18\t \x01(\x0b\x32#.envoy.config.trace.v3.Tracing.Http\x12\x37\n\x13spawn_upstream_span\x18\n \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x11\n\toperation\x18\x0b \x01(\t\x12\x1a\n\x12upstream_operation\x18\x0c \x01(\t\"(\n\rOperationName\x12\x0b\n\x07INGRESS\x10\x00\x12\n\n\x06\x45GRESS\x10\x01:[\x9a\xc5\x88\x1eV\nTenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.TracingJ\x04\x08\x01\x10\x02J\x04\x08\x02\x10\x03R\x0eoperation_nameR\x18request_headers_for_tags\x1a\xce\x01\n\x15InternalAddressConfig\x12\x14\n\x0cunix_sockets\x18\x01 \x01(\x08\x12\x34\n\x0b\x63idr_ranges\x18\x02 \x03(\x0b\x32\x1f.envoy.config.core.v3.CidrRange:i\x9a\xc5\x88\x1e\x64\nbenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.InternalAddressConfig\x1a\xf8\x01\n\x1bSetCurrentClientCertDetails\x12+\n\x07subject\x18\x01 \x01(\x0b\x32\x1a.google.protobuf.BoolValue\x12\x0c\n\x04\x63\x65rt\x18\x03 \x01(\x08\x12\r\n\x05\x63hain\x18\x06 \x01(\x08\x12\x0b\n\x03\x64ns\x18\x04 \x01(\x08\x12\x0b\n\x03uri\x18\x05 \x01(\x08:o\x9a\xc5\x88\x1ej\nhenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.SetCurrentClientCertDetailsJ\x04\x08\x02\x10\x03\x1a\xc6\x02\n\x17\x46orwardClientCertConfig\x12\x90\x01\n\x1b\x66orward_client_cert_details\x18\x01 \x01(\x0e\x32k.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.ForwardClientCertDetails\x12\x97\x01\n\x1fset_current_client_cert_details\x18\x02 \x01(\x0b\x32n.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManager.SetCurrentClientCertDetails\x1a\x8f\x02\n\rUpgradeConfig\x12\x14\n\x0cupgrade_type\x18\x01 \x01(\t\x12X\n\x07\x66ilters\x18\x02 \x03(\x0b\x32G.envoy.extensions.filters.network.http_connection_manager.v3.HttpFilter\x12+\n\x07\x65nabled\x18\x03 \x01(\x0b\x32\x1a.google.protobuf.BoolValue:a\x9a\xc5\x88\x1e\\\nZenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.UpgradeConfig\x1a\xb1\x01\n\x18PathNormalizationOptions\x12I\n\x19\x66orwarding_transformation\x18\x01 \x01(\x0b\x32&.envoy.type.http.v3.PathTransformation\x12J\n\x1ahttp_filter_transformation\x18\x02 \x01(\x0b\x32&.envoy.type.http.v3.PathTransformation\x1a\xe3\x01\n\x11ProxyStatusConfig\x12\x16\n\x0eremove_details\x18\x01 \x01(\x08\x12-\n%remove_connection_termination_details\x18\x02 \x01(\x08\x12\x1d\n\x15remove_response_flags\x18\x03 \x01(\x08\x12%\n\x1dset_recommended_response_code\x18\x04 \x01(\x08\x12\x15\n\x0buse_node_id\x18\x05 \x01(\x08H\x00\x12\x1c\n\x12literal_proxy_name\x18\x06 \x01(\tH\x00\x42\x0c\n\nproxy_name\x1a\xc0\x01\n\x13HcmAccessLogOptions\x12J\n\x19\x61\x63\x63\x65ss_log_flush_interval\x18\x01 \x01(\x0b\x32\x19.google.protobuf.DurationB\x0c\xfa\x42\t\xaa\x01\x06\x32\x04\x10\xc0\x84=\x12\'\n\x1f\x66lush_access_log_on_new_request\x18\x02 \x01(\x08\x12\x34\n,flush_log_on_tunnel_successfully_established\x18\x03 \x01(\x08\"6\n\tCodecType\x12\x08\n\x04\x41UTO\x10\x00\x12\t\n\x05HTTP1\x10\x01\x12\t\n\x05HTTP2\x10\x02\x12\t\n\x05HTTP3\x10\x03\"S\n\x1aServerHeaderTransformation\x12\r\n\tOVERWRITE\x10\x00\x12\x14\n\x10\x41PPEND_IF_ABSENT\x10\x01\x12\x10\n\x0cPASS_THROUGH\x10\x02\"y\n\x18\x46orwardClientCertDetails\x12\x0c\n\x08SANITIZE\x10\x00\x12\x10\n\x0c\x46ORWARD_ONLY\x10\x01\x12\x12\n\x0e\x41PPEND_FORWARD\x10\x02\x12\x10\n\x0cSANITIZE_SET\x10\x03\x12\x17\n\x13\x41LWAYS_FORWARD_ONLY\x10\x04\"\xa0\x01\n\x1cPathWithEscapedSlashesAction\x12#\n\x1fIMPLEMENTATION_SPECIFIC_DEFAULT\x10\x00\x12\x12\n\x0eKEEP_UNCHANGED\x10\x01\x12\x12\n\x0eREJECT_REQUEST\x10\x02\x12\x19\n\x15UNESCAPE_AND_REDIRECT\x10\x03\x12\x18\n\x14UNESCAPE_AND_FORWARD\x10\x04:S\x9a\xc5\x88\x1eN\nLenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManagerB\x16\n\x0froute_specifier\x12\x03\xf8\x42\x01\x42\x11\n\x0fstrip_port_modeJ\x04\x08\x1b\x10\x1cJ\x04\x08\x0b\x10\x0cR\x0cidle_timeout\"\xb5\x01\n\x10LocalReplyConfig\x12\\\n\x07mappers\x18\x01 \x03(\x0b\x32K.envoy.extensions.filters.network.http_connection_manager.v3.ResponseMapper\x12\x43\n\x0b\x62ody_format\x18\x02 \x01(\x0b\x32..envoy.config.core.v3.SubstitutionFormatString\"\xe0\x02\n\x0eResponseMapper\x12\x44\n\x06\x66ilter\x18\x01 \x01(\x0b\x32*.envoy.config.accesslog.v3.AccessLogFilterB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12>\n\x0bstatus_code\x18\x02 \x01(\x0b\x32\x1c.google.protobuf.UInt32ValueB\x0b\xfa\x42\x08*\x06\x10\xd8\x04(\xc8\x01\x12.\n\x04\x62ody\x18\x03 \x01(\x0b\x32 .envoy.config.core.v3.DataSource\x12L\n\x14\x62ody_format_override\x18\x04 \x01(\x0b\x32..envoy.config.core.v3.SubstitutionFormatString\x12J\n\x0eheaders_to_add\x18\x05 \x03(\x0b\x32\'.envoy.config.core.v3.HeaderValueOptionB\t\xfa\x42\x06\x92\x01\x03\x10\xe8\x07\"\x9e\x01\n\x03Rds\x12\x39\n\rconfig_source\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSource\x12\x19\n\x11route_config_name\x18\x02 \x01(\t:A\x9a\xc5\x88\x1e<\n:envoy.config.filter.network.http_connection_manager.v2.Rds\"\xdc\x01\n\x1dScopedRouteConfigurationsList\x12^\n\x1bscoped_route_configurations\x18\x01 \x03(\x0b\x32/.envoy.config.route.v3.ScopedRouteConfigurationB\x08\xfa\x42\x05\x92\x01\x02\x08\x01:[\x9a\xc5\x88\x1eV\nTenvoy.config.filter.network.http_connection_manager.v2.ScopedRouteConfigurationsList\"\xb9\r\n\x0cScopedRoutes\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12~\n\x11scope_key_builder\x18\x02 \x01(\x0b\x32Y.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRoutes.ScopeKeyBuilderB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12=\n\x11rds_config_source\x18\x03 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSource\x12\x86\x01\n scoped_route_configurations_list\x18\x04 \x01(\x0b\x32Z.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRouteConfigurationsListH\x00\x12\\\n\nscoped_rds\x18\x05 \x01(\x0b\x32\x46.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRdsH\x00\x1a\x86\t\n\x0fScopeKeyBuilder\x12\x86\x01\n\tfragments\x18\x01 \x03(\x0b\x32i.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRoutes.ScopeKeyBuilder.FragmentBuilderB\x08\xfa\x42\x05\x92\x01\x02\x08\x01\x1a\x8d\x07\n\x0f\x46ragmentBuilder\x12\xa0\x01\n\x16header_value_extractor\x18\x01 \x01(\x0b\x32~.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractorH\x00\x1a\xdd\x04\n\x14HeaderValueExtractor\x12\x1b\n\x04name\x18\x01 \x01(\tB\r\xfa\x42\nr\x08\x10\x01\xc0\x01\x01\xc8\x01\x00\x12\x19\n\x11\x65lement_separator\x18\x02 \x01(\t\x12\x0f\n\x05index\x18\x03 \x01(\rH\x00\x12\x9c\x01\n\x07\x65lement\x18\x04 \x01(\x0b\x32\x88\x01.envoy.extensions.filters.network.http_connection_manager.v3.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractor.KvElementH\x00\x1a\xcb\x01\n\tKvElement\x12\x1a\n\tseparator\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x14\n\x03key\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01:\x8b\x01\x9a\xc5\x88\x1e\x85\x01\n\x82\x01\x65nvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractor.KvElement:\x7f\x9a\xc5\x88\x1ez\nxenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractorB\x0e\n\x0c\x65xtract_type:j\x9a\xc5\x88\x1e\x65\ncenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilderB\x0b\n\x04type\x12\x03\xf8\x42\x01:Z\x9a\xc5\x88\x1eU\nSenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder:J\x9a\xc5\x88\x1e\x45\nCenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutesB\x17\n\x10\x63onfig_specifier\x12\x03\xf8\x42\x01\"\xc4\x01\n\tScopedRds\x12N\n\x18scoped_rds_config_source\x18\x01 \x01(\x0b\x32\".envoy.config.core.v3.ConfigSourceB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x1e\n\x16srds_resources_locator\x18\x02 \x01(\t:G\x9a\xc5\x88\x1e\x42\n@envoy.config.filter.network.http_connection_manager.v2.ScopedRds\"\xae\x02\n\nHttpFilter\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12,\n\x0ctyped_config\x18\x04 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x12G\n\x10\x63onfig_discovery\x18\x05 \x01(\x0b\x32+.envoy.config.core.v3.ExtensionConfigSourceH\x00\x12\x13\n\x0bis_optional\x18\x06 \x01(\x08\x12\x10\n\x08\x64isabled\x18\x07 \x01(\x08:H\x9a\xc5\x88\x1e\x43\nAenvoy.config.filter.network.http_connection_manager.v2.HttpFilterB\r\n\x0b\x63onfig_typeJ\x04\x08\x03\x10\x04J\x04\x08\x02\x10\x03R\x06\x63onfig\"\x92\x01\n\x12RequestIDExtension\x12*\n\x0ctyped_config\x18\x01 \x01(\x0b\x32\x14.google.protobuf.Any:P\x9a\xc5\x88\x1eK\nIenvoy.config.filter.network.http_connection_manager.v2.RequestIDExtension\"\x86\x01\n EnvoyMobileHttpConnectionManager\x12\x62\n\x06\x63onfig\x18\x01 \x01(\x0b\x32R.envoy.extensions.filters.network.http_connection_manager.v3.HttpConnectionManagerB\xef\x01\nIio.envoyproxy.envoy.extensions.filters.network.http_connection_manager.v3B\x1aHttpConnectionManagerProtoP\x01Z|github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3;http_connection_managerv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.filters.network.http_connection_manager.v3.http_connection_manager_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\nIio.envoyproxy.envoy.extensions.filters.network.http_connection_manager.v3B\032HttpConnectionManagerProtoP\001Z|github.com/envoyproxy/go-control-plane/envoy/extensions/filters/network/http_connection_manager/v3;http_connection_managerv3\272\200\310\321\006\002\020\002'
  _globals['_HTTPCONNECTIONMANAGER_TRACING']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER_TRACING']._serialized_options = b'\232\305\210\036V\nTenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.Tracing'
  _globals['_HTTPCONNECTIONMANAGER_INTERNALADDRESSCONFIG']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER_INTERNALADDRESSCONFIG']._serialized_options = b'\232\305\210\036d\nbenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.InternalAddressConfig'
  _globals['_HTTPCONNECTIONMANAGER_SETCURRENTCLIENTCERTDETAILS']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER_SETCURRENTCLIENTCERTDETAILS']._serialized_options = b'\232\305\210\036j\nhenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.SetCurrentClientCertDetails'
  _globals['_HTTPCONNECTIONMANAGER_UPGRADECONFIG']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER_UPGRADECONFIG']._serialized_options = b'\232\305\210\036\\\nZenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager.UpgradeConfig'
  _globals['_HTTPCONNECTIONMANAGER_HCMACCESSLOGOPTIONS'].fields_by_name['access_log_flush_interval']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER_HCMACCESSLOGOPTIONS'].fields_by_name['access_log_flush_interval']._serialized_options = b'\372B\t\252\001\0062\004\020\300\204='
  _globals['_HTTPCONNECTIONMANAGER'].oneofs_by_name['route_specifier']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].oneofs_by_name['route_specifier']._serialized_options = b'\370B\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['codec_type']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['codec_type']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['common_http_protocol_options']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['common_http_protocol_options']._serialized_options = b'\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['http2_protocol_options']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['http2_protocol_options']._serialized_options = b'\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['server_name']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['server_name']._serialized_options = b'\372B\010r\006\300\001\002\310\001\000'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['server_header_transformation']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['server_header_transformation']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['max_request_headers_kb']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['max_request_headers_kb']._serialized_options = b'\372B\007*\005\030\200@ \000'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['stream_idle_timeout']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['stream_idle_timeout']._serialized_options = b'\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['request_timeout']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['request_timeout']._serialized_options = b'\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['request_headers_timeout']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['request_headers_timeout']._serialized_options = b'\372B\005\252\001\0022\000\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['access_log_flush_interval']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['access_log_flush_interval']._serialized_options = b'\030\001\372B\t\252\001\0062\004\020\300\204=\222\307\206\330\004\0033.0'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['flush_access_log_on_new_request']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['flush_access_log_on_new_request']._serialized_options = b'\030\001\222\307\206\330\004\0033.0'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['use_remote_address']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['use_remote_address']._serialized_options = b'\212\223\267*\002\010\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['via']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['via']._serialized_options = b'\372B\010r\006\300\001\002\310\001\000'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['forward_client_cert_details']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['forward_client_cert_details']._serialized_options = b'\372B\005\202\001\002\020\001'
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['strip_matching_host_port']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER'].fields_by_name['strip_matching_host_port']._serialized_options = b'\362\230\376\217\005\021\022\017strip_port_mode'
  _globals['_HTTPCONNECTIONMANAGER']._loaded_options = None
  _globals['_HTTPCONNECTIONMANAGER']._serialized_options = b'\232\305\210\036N\nLenvoy.config.filter.network.http_connection_manager.v2.HttpConnectionManager'
  _globals['_RESPONSEMAPPER'].fields_by_name['filter']._loaded_options = None
  _globals['_RESPONSEMAPPER'].fields_by_name['filter']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_RESPONSEMAPPER'].fields_by_name['status_code']._loaded_options = None
  _globals['_RESPONSEMAPPER'].fields_by_name['status_code']._serialized_options = b'\372B\010*\006\020\330\004(\310\001'
  _globals['_RESPONSEMAPPER'].fields_by_name['headers_to_add']._loaded_options = None
  _globals['_RESPONSEMAPPER'].fields_by_name['headers_to_add']._serialized_options = b'\372B\006\222\001\003\020\350\007'
  _globals['_RDS']._loaded_options = None
  _globals['_RDS']._serialized_options = b'\232\305\210\036<\n:envoy.config.filter.network.http_connection_manager.v2.Rds'
  _globals['_SCOPEDROUTECONFIGURATIONSLIST'].fields_by_name['scoped_route_configurations']._loaded_options = None
  _globals['_SCOPEDROUTECONFIGURATIONSLIST'].fields_by_name['scoped_route_configurations']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_SCOPEDROUTECONFIGURATIONSLIST']._loaded_options = None
  _globals['_SCOPEDROUTECONFIGURATIONSLIST']._serialized_options = b'\232\305\210\036V\nTenvoy.config.filter.network.http_connection_manager.v2.ScopedRouteConfigurationsList'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT'].fields_by_name['separator']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT'].fields_by_name['separator']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT'].fields_by_name['key']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT'].fields_by_name['key']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT']._serialized_options = b'\232\305\210\036\205\001\n\202\001envoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractor.KvElement'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR'].fields_by_name['name']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR'].fields_by_name['name']._serialized_options = b'\372B\nr\010\020\001\300\001\001\310\001\000'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR']._serialized_options = b'\232\305\210\036z\nxenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder.HeaderValueExtractor'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER'].oneofs_by_name['type']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER'].oneofs_by_name['type']._serialized_options = b'\370B\001'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER']._serialized_options = b'\232\305\210\036e\ncenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder.FragmentBuilder'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER'].fields_by_name['fragments']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER'].fields_by_name['fragments']._serialized_options = b'\372B\005\222\001\002\010\001'
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER']._loaded_options = None
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER']._serialized_options = b'\232\305\210\036U\nSenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes.ScopeKeyBuilder'
  _globals['_SCOPEDROUTES'].oneofs_by_name['config_specifier']._loaded_options = None
  _globals['_SCOPEDROUTES'].oneofs_by_name['config_specifier']._serialized_options = b'\370B\001'
  _globals['_SCOPEDROUTES'].fields_by_name['name']._loaded_options = None
  _globals['_SCOPEDROUTES'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_SCOPEDROUTES'].fields_by_name['scope_key_builder']._loaded_options = None
  _globals['_SCOPEDROUTES'].fields_by_name['scope_key_builder']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_SCOPEDROUTES']._loaded_options = None
  _globals['_SCOPEDROUTES']._serialized_options = b'\232\305\210\036E\nCenvoy.config.filter.network.http_connection_manager.v2.ScopedRoutes'
  _globals['_SCOPEDRDS'].fields_by_name['scoped_rds_config_source']._loaded_options = None
  _globals['_SCOPEDRDS'].fields_by_name['scoped_rds_config_source']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_SCOPEDRDS']._loaded_options = None
  _globals['_SCOPEDRDS']._serialized_options = b'\232\305\210\036B\n@envoy.config.filter.network.http_connection_manager.v2.ScopedRds'
  _globals['_HTTPFILTER'].fields_by_name['name']._loaded_options = None
  _globals['_HTTPFILTER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_HTTPFILTER']._loaded_options = None
  _globals['_HTTPFILTER']._serialized_options = b'\232\305\210\036C\nAenvoy.config.filter.network.http_connection_manager.v2.HttpFilter'
  _globals['_REQUESTIDEXTENSION']._loaded_options = None
  _globals['_REQUESTIDEXTENSION']._serialized_options = b'\232\305\210\036K\nIenvoy.config.filter.network.http_connection_manager.v2.RequestIDExtension'
  _globals['_HTTPCONNECTIONMANAGER']._serialized_start=991
  _globals['_HTTPCONNECTIONMANAGER']._serialized_end=8315
  _globals['_HTTPCONNECTIONMANAGER_TRACING']._serialized_start=5430
  _globals['_HTTPCONNECTIONMANAGER_TRACING']._serialized_end=6066
  _globals['_HTTPCONNECTIONMANAGER_TRACING_OPERATIONNAME']._serialized_start=5879
  _globals['_HTTPCONNECTIONMANAGER_TRACING_OPERATIONNAME']._serialized_end=5919
  _globals['_HTTPCONNECTIONMANAGER_INTERNALADDRESSCONFIG']._serialized_start=6069
  _globals['_HTTPCONNECTIONMANAGER_INTERNALADDRESSCONFIG']._serialized_end=6275
  _globals['_HTTPCONNECTIONMANAGER_SETCURRENTCLIENTCERTDETAILS']._serialized_start=6278
  _globals['_HTTPCONNECTIONMANAGER_SETCURRENTCLIENTCERTDETAILS']._serialized_end=6526
  _globals['_HTTPCONNECTIONMANAGER_FORWARDCLIENTCERTCONFIG']._serialized_start=6529
  _globals['_HTTPCONNECTIONMANAGER_FORWARDCLIENTCERTCONFIG']._serialized_end=6855
  _globals['_HTTPCONNECTIONMANAGER_UPGRADECONFIG']._serialized_start=6858
  _globals['_HTTPCONNECTIONMANAGER_UPGRADECONFIG']._serialized_end=7129
  _globals['_HTTPCONNECTIONMANAGER_PATHNORMALIZATIONOPTIONS']._serialized_start=7132
  _globals['_HTTPCONNECTIONMANAGER_PATHNORMALIZATIONOPTIONS']._serialized_end=7309
  _globals['_HTTPCONNECTIONMANAGER_PROXYSTATUSCONFIG']._serialized_start=7312
  _globals['_HTTPCONNECTIONMANAGER_PROXYSTATUSCONFIG']._serialized_end=7539
  _globals['_HTTPCONNECTIONMANAGER_HCMACCESSLOGOPTIONS']._serialized_start=7542
  _globals['_HTTPCONNECTIONMANAGER_HCMACCESSLOGOPTIONS']._serialized_end=7734
  _globals['_HTTPCONNECTIONMANAGER_CODECTYPE']._serialized_start=7736
  _globals['_HTTPCONNECTIONMANAGER_CODECTYPE']._serialized_end=7790
  _globals['_HTTPCONNECTIONMANAGER_SERVERHEADERTRANSFORMATION']._serialized_start=7792
  _globals['_HTTPCONNECTIONMANAGER_SERVERHEADERTRANSFORMATION']._serialized_end=7875
  _globals['_HTTPCONNECTIONMANAGER_FORWARDCLIENTCERTDETAILS']._serialized_start=7877
  _globals['_HTTPCONNECTIONMANAGER_FORWARDCLIENTCERTDETAILS']._serialized_end=7998
  _globals['_HTTPCONNECTIONMANAGER_PATHWITHESCAPEDSLASHESACTION']._serialized_start=8001
  _globals['_HTTPCONNECTIONMANAGER_PATHWITHESCAPEDSLASHESACTION']._serialized_end=8161
  _globals['_LOCALREPLYCONFIG']._serialized_start=8318
  _globals['_LOCALREPLYCONFIG']._serialized_end=8499
  _globals['_RESPONSEMAPPER']._serialized_start=8502
  _globals['_RESPONSEMAPPER']._serialized_end=8854
  _globals['_RDS']._serialized_start=8857
  _globals['_RDS']._serialized_end=9015
  _globals['_SCOPEDROUTECONFIGURATIONSLIST']._serialized_start=9018
  _globals['_SCOPEDROUTECONFIGURATIONSLIST']._serialized_end=9238
  _globals['_SCOPEDROUTES']._serialized_start=9241
  _globals['_SCOPEDROUTES']._serialized_end=10962
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER']._serialized_start=9703
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER']._serialized_end=10861
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER']._serialized_start=9860
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER']._serialized_end=10769
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR']._serialized_start=10043
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR']._serialized_end=10648
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT']._serialized_start=10300
  _globals['_SCOPEDROUTES_SCOPEKEYBUILDER_FRAGMENTBUILDER_HEADERVALUEEXTRACTOR_KVELEMENT']._serialized_end=10503
  _globals['_SCOPEDRDS']._serialized_start=10965
  _globals['_SCOPEDRDS']._serialized_end=11161
  _globals['_HTTPFILTER']._serialized_start=11164
  _globals['_HTTPFILTER']._serialized_end=11466
  _globals['_REQUESTIDEXTENSION']._serialized_start=11469
  _globals['_REQUESTIDEXTENSION']._serialized_end=11615
  _globals['_ENVOYMOBILEHTTPCONNECTIONMANAGER']._serialized_start=11618
  _globals['_ENVOYMOBILEHTTPCONNECTIONMANAGER']._serialized_end=11752
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: google/rpc/http.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'google/rpc/http.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x15google/rpc/http.proto\x12\ngoogle.rpc\"a\n\x0bHttpRequest\x12\x0e\n\x06method\x18\x01 \x01(\t\x12\x0b\n\x03uri\x18\x02 \x01(\t\x12\'\n\x07headers\x18\x03 \x03(\x0b\x32\x16.google.rpc.HttpHeader\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\"e\n\x0cHttpResponse\x12\x0e\n\x06status\x18\x01 \x01(\x05\x12\x0e\n\x06reason\x18\x02 \x01(\t\x12\'\n\x07headers\x18\x03 \x03(\x0b\x32\x16.google.rpc.HttpHeader\x12\x0c\n\x04\x62ody\x18\x04 \x01(\x0c\"(\n\nHttpHeader\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\tBX\n\x0e\x63om.google.rpcB\tHttpProtoP\x01Z3google.golang.org/genproto/googleapis/rpc/http;http\xa2\x02\x03RPCb\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'google.rpc.http_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n\016com.google.rpcB\tHttpProtoP\001Z3google.golang.org/genproto/googleapis/rpc/http;http\242\002\003RPC'
  _globals['_HTTPREQUEST']._serialized_start=37
  _globals['_HTTPREQUEST']._serialized_end=134
  _globals['_HTTPRESPONSE']._serialized_start=136
  _globals['_HTTPRESPONSE']._serialized_end=237
  _globals['_HTTPHEADER']._serialized_start=239
  _globals['_HTTPHEADER']._serialized_end=279
# @@protoc_insertion_point(module_scope)



#!/usr/bin/env python3

# Copyright 2024 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


# Explicitly ban select functions from being used in gRPC.
# Any new instance of a deprecated function being used in the code will be
# flagged by the script. If there is a new instance of a deprecated function in
# a Pull Request, then the Sanity tests will fail for the Pull Request.
# The allow list has a list of files where clean up of deprecated functions is
# pending.

import os
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

# More files may be added to the RUBY_PHP_ALLOW_LIST
# if they belong to the PHP or RUBY folder.
RUBY_PHP_ALLOW_LIST = [
    "./include/grpc/support/log.h",
    "./src/core/util/log.cc",
    "./src/php/ext/grpc/call_credentials.c",
    "./src/php/ext/grpc/channel.c",
    "./src/ruby/ext/grpc/rb_call.c",
    "./src/ruby/ext/grpc/rb_call_credentials.c",
    "./src/ruby/ext/grpc/rb_channel.c",
    "./src/ruby/ext/grpc/rb_completion_queue.c",
    "./src/ruby/ext/grpc/rb_event_thread.c",
    "./src/ruby/ext/grpc/rb_grpc.c",
    "./src/ruby/ext/grpc/rb_server.c",
]

#  Map of deprecated functions to allowlist files
DEPRECATED_FUNCTION_TEMP_ALLOW_LIST = {
    # These experimental logging functions are only for php and ruby.
    "grpc_absl_log(": RUBY_PHP_ALLOW_LIST,
    "grpc_absl_log_int(": RUBY_PHP_ALLOW_LIST,
    "grpc_absl_log_str(": RUBY_PHP_ALLOW_LIST,
    # These have been deprecated.
    # Most of these have been deleted.
    # Putting this check here just to prevent people from
    # submitting PRs with any of these commented out.
    "gpr_assertion_failed": [],  # Safe to delete this entry after Nov 2024.
    "gpr_log(": [],  # Safe to delete this entry after Nov 2024.
    "gpr_log_func_args": [],  # Safe to delete this entry after Nov 2024.
    "gpr_log_message": [],  # Safe to delete this entry after Nov 2024.
    "gpr_log_severity_string": [],  # Safe to delete this entry after Nov 2024.
    "gpr_set_log_function": [],  # Safe to delete this entry after Nov 2024.
    "gpr_set_log_verbosity": [],  # Safe to delete this entry after Nov 2024.
    "gpr_should_log": [],  # Safe to delete this entry after Nov 2024.
    "GPR_ASSERT": [],  # Safe to delete this entry after Nov 2024.
    "GPR_DEBUG_ASSERT": [],  # Safe to delete this entry after Nov 2024.
}

errors = 0
num_files = 0
for root, dirs, files in os.walk("."):
    for filename in files:
        num_files += 1
        path = os.path.join(root, filename)
        if os.path.splitext(path)[1] not in (".h", ".cc", ".c"):
            continue
        with open(path) as f:
            text = f.read()
        for deprecated, allowlist in list(
            DEPRECATED_FUNCTION_TEMP_ALLOW_LIST.items()
        ):
            if path in allowlist:
                continue
            if deprecated in text:
                print(
                    (
                        'Illegal use of "%s" in %s. Use absl functions instead.'
                        % (deprecated, path)
                    )
                )
                errors += 1

assert errors == 0
if errors > 0:
    print(("Number of errors : %d " % (errors)))

# This check comes about from this issue:
# https://github.com/grpc/grpc/issues/15381
# Basically, a change rendered this script useless and we did not realize it.
# This check ensures that this type of issue doesn't occur again.
assert num_files > 18000
print("Number of files checked : %d " % (num_files))



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.


cdef class _GrpcArgWrapper:

  cdef grpc_arg arg


cdef tuple _wrap_grpc_arg(grpc_arg arg):
  wrapped = _GrpcArgWrapper()
  wrapped.arg = arg
  return ("grpc.python._cygrpc._GrpcArgWrapper", wrapped)


cdef grpc_arg _unwrap_grpc_arg(tuple wrapped_arg):
  cdef _GrpcArgWrapper wrapped = wrapped_arg[1]
  return wrapped.arg


cdef class _ChannelArg:

  cdef void c(self, argument, references) except *:
    key, value = argument
    cdef bytes encoded_key = _encode(key)
    if encoded_key is not key:
      references.append(encoded_key)
    self.c_argument.key = encoded_key
    if isinstance(value, int):
      self.c_argument.type = GRPC_ARG_INTEGER
      self.c_argument.value.integer = value
    elif isinstance(value, (bytes, str, unicode,)):
      self.c_argument.type = GRPC_ARG_STRING
      encoded_value = _encode(value)
      if encoded_value is not value:
        references.append(encoded_value)
      self.c_argument.value.string = encoded_value
    elif isinstance(value, _GrpcArgWrapper):
      self.c_argument = (<_GrpcArgWrapper>value).arg
    elif isinstance(value, tuple) and len(value) == 2 and value[0] == "grpc.python._cygrpc._GrpcArgWrapper":
      self.c_argument = _unwrap_grpc_arg(value)
    elif hasattr(value, '__int__'):
      # Pointer objects must override __int__() to return
      # the underlying C address (Python ints are word size). The
      # lifecycle of the pointer is fixed to the lifecycle of the
      # python object wrapping it.
      self.c_argument.type = GRPC_ARG_POINTER
      self.c_argument.value.pointer.vtable = &default_vtable
      self.c_argument.value.pointer.address = <void*>(<uintptr_t>int(value))
    else:
      raise TypeError(
          'Expected int, bytes, or behavior, got {}'.format(type(value)))


cdef class _ChannelArgs:

  def __cinit__(self, arguments):
    self._arguments = () if arguments is None else tuple(arguments)
    self._channel_args = []
    self._references = []
    self._c_arguments.arguments_length = len(self._arguments)
    if self._c_arguments.arguments_length != 0:
      self._c_arguments.arguments = <grpc_arg *>gpr_malloc(
          self._c_arguments.arguments_length * sizeof(grpc_arg))
      for index, argument in enumerate(self._arguments):
        channel_arg = _ChannelArg()
        channel_arg.c(argument, self._references)
        self._c_arguments.arguments[index] = channel_arg.c_argument
        self._channel_args.append(channel_arg)

  cdef grpc_channel_args *c_args(self) except *:
    return &self._c_arguments

  def __dealloc__(self):
    if self._c_arguments.arguments != NULL:
      gpr_free(self._c_arguments.arguments)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import threading
import time
import unittest

from grpc._cython import cygrpc

from tests.unit.framework.common import test_constants


def _channel():
    return cygrpc.Channel(b"localhost:54321", (), None)


def _connectivity_loop(channel):
    for _ in range(100):
        connectivity = channel.check_connectivity_state(True)
        channel.watch_connectivity_state(connectivity, time.time() + 0.2)


def _create_loop_destroy():
    channel = _channel()
    _connectivity_loop(channel)
    channel.close(cygrpc.StatusCode.ok, "Channel close!")


def _in_parallel(behavior, arguments):
    threads = tuple(
        threading.Thread(target=behavior, args=arguments)
        for _ in range(test_constants.THREAD_CONCURRENCY)
    )
    for thread in threads:
        thread.start()
    for thread in threads:
        thread.join()


class ChannelTest(unittest.TestCase):
    def test_single_channel_lonely_connectivity(self):
        channel = _channel()
        _connectivity_loop(channel)
        channel.close(cygrpc.StatusCode.ok, "Channel close!")

    def test_multiple_channels_lonely_connectivity(self):
        _in_parallel(_create_loop_destroy, ())

    def test_negative_deadline_connectivity(self):
        channel = _channel()
        connectivity = channel.check_connectivity_state(True)
        channel.watch_connectivity_state(connectivity, -3.14)
        channel.close(cygrpc.StatusCode.ok, "Channel close!")
        # NOTE(lidiz) The negative timeout should not trigger SIGABRT.
        # Bug report: https://github.com/grpc/grpc/issues/18244


if __name__ == "__main__":
    unittest.main(verbosity=2)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: bazel/cc_proto_descriptor_library/testdata/test.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'bazel/cc_proto_descriptor_library/testdata/test.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n5bazel/cc_proto_descriptor_library/testdata/test.proto\x12\x1ctestdata.dynamic_descriptors\x1a\x19google/protobuf/any.proto\";\n\x03\x46oo\x12\x0b\n\x03\x62\x61r\x18\x01 \x01(\t\x12!\n\x03\x62\x61z\x18\x02 \x01(\x0b\x32\x14.google.protobuf.Any*\x04\x08\n\x10\x15')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'bazel.cc_proto_descriptor_library.testdata.test_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  DESCRIPTOR._loaded_options = None
  _globals['_FOO']._serialized_start=114
  _globals['_FOO']._serialized_end=173
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: udpa/annotations/security.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'udpa/annotations/security.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from google.protobuf import descriptor_pb2 as google_dot_protobuf_dot_descriptor__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x1fudpa/annotations/security.proto\x12\x10udpa.annotations\x1a\x1dudpa/annotations/status.proto\x1a google/protobuf/descriptor.proto\"o\n\x17\x46ieldSecurityAnnotation\x12*\n\"configure_for_untrusted_downstream\x18\x01 \x01(\x08\x12(\n configure_for_untrusted_upstream\x18\x02 \x01(\x08:]\n\x08security\x12\x1d.google.protobuf.FieldOptions\x18\xb1\xf2\xa6\x05 \x01(\x0b\x32).udpa.annotations.FieldSecurityAnnotationB1Z\'github.com/cncf/xds/go/udpa/annotations\xba\x80\xc8\xd1\x06\x02\x08\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'udpa.annotations.security_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'Z\'github.com/cncf/xds/go/udpa/annotations\272\200\310\321\006\002\010\001'
  _globals['_FIELDSECURITYANNOTATION']._serialized_start=118
  _globals['_FIELDSECURITYANNOTATION']._serialized_end=229
# @@protoc_insertion_point(module_scope)



# Copyright 2019 The gRPC Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
import asyncio
import logging
import unittest

import grpc
from grpc.experimental import aio
import typeguard

from src.proto.grpc.testing import messages_pb2
from src.proto.grpc.testing import test_pb2_grpc
from tests_aio.unit import _common
from tests_aio.unit import _constants
from tests_aio.unit._test_base import AioTestBase
from tests_aio.unit._test_server import _INITIAL_METADATA_KEY
from tests_aio.unit._test_server import _TRAILING_METADATA_KEY
from tests_aio.unit._test_server import start_test_server

_LOCAL_CANCEL_DETAILS_EXPECTATION = "Locally cancelled by application!"
_INITIAL_METADATA_TO_INJECT = aio.Metadata(
    (_INITIAL_METADATA_KEY, "extra info"),
    (_TRAILING_METADATA_KEY, b"\x13\x37"),
)
_TIMEOUT_CHECK_IF_CALLBACK_WAS_CALLED = 1.0


class TestUnaryUnaryClientInterceptor(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    @typeguard.suppress_type_checks  # testing negative cases
    def test_invalid_interceptor(self):
        class InvalidInterceptor:
            """Just an invalid Interceptor"""

        with self.assertRaises(ValueError):
            aio.insecure_channel("", interceptors=[InvalidInterceptor()])

    async def test_executed_right_order(self):
        interceptors_executed = []

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            """Interceptor used for testing if the interceptor is being called"""

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                interceptors_executed.append(self)
                call = await continuation(client_call_details, request)
                return call

        interceptors = [Interceptor() for i in range(2)]

        async with aio.insecure_channel(
            self._server_target, interceptors=interceptors
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            response = await call

            # Check that all interceptors were executed, and were executed
            # in the right order.
            self.assertSequenceEqual(interceptors_executed, interceptors)

            self.assertIsInstance(response, messages_pb2.SimpleResponse)

    @unittest.expectedFailure
    # TODO(https://github.com/grpc/grpc/issues/20144) Once metadata support is
    # implemented in the client-side, this test must be implemented.
    def test_modify_metadata(self):
        raise NotImplementedError()

    @unittest.expectedFailure
    # TODO(https://github.com/grpc/grpc/issues/20532) Once credentials support is
    # implemented in the client-side, this test must be implemented.
    def test_modify_credentials(self):
        raise NotImplementedError()

    async def test_status_code_Ok(self):
        class StatusCodeOkInterceptor(aio.UnaryUnaryClientInterceptor):
            """Interceptor used for observing status code Ok returned by the RPC"""

            def __init__(self):
                self.status_code_Ok_observed = False

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                code = await call.code()
                if code == grpc.StatusCode.OK:
                    self.status_code_Ok_observed = True

                return call

        interceptor = StatusCodeOkInterceptor()

        async with aio.insecure_channel(
            self._server_target, interceptors=[interceptor]
        ) as channel:
            # when no error StatusCode.OK must be observed
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            await multicallable(messages_pb2.SimpleRequest())

            self.assertTrue(interceptor.status_code_Ok_observed)

    async def test_add_timeout(self):
        class TimeoutInterceptor(aio.UnaryUnaryClientInterceptor):
            """Interceptor used for adding a timeout to the RPC"""

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                new_client_call_details = aio.ClientCallDetails(
                    method=client_call_details.method,
                    timeout=_constants.UNARY_CALL_WITH_SLEEP_VALUE / 2,
                    metadata=client_call_details.metadata,
                    credentials=client_call_details.credentials,
                    wait_for_ready=client_call_details.wait_for_ready,
                )
                return await continuation(new_client_call_details, request)

        interceptor = TimeoutInterceptor()

        async with aio.insecure_channel(
            self._server_target, interceptors=[interceptor]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCallWithSleep",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(messages_pb2.SimpleRequest())

            with self.assertRaises(aio.AioRpcError) as exception_context:
                await call

            self.assertEqual(
                exception_context.exception.code(),
                grpc.StatusCode.DEADLINE_EXCEEDED,
            )

            self.assertTrue(call.done())
            self.assertEqual(
                grpc.StatusCode.DEADLINE_EXCEEDED, await call.code()
            )

    async def test_retry(self):
        class RetryInterceptor(aio.UnaryUnaryClientInterceptor):
            """Simulates a Retry Interceptor which ends up by making
            two RPC calls."""

            def __init__(self):
                self.calls = []

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                new_client_call_details = aio.ClientCallDetails(
                    method=client_call_details.method,
                    timeout=_constants.UNARY_CALL_WITH_SLEEP_VALUE / 2,
                    metadata=client_call_details.metadata,
                    credentials=client_call_details.credentials,
                    wait_for_ready=client_call_details.wait_for_ready,
                )

                try:
                    call = await continuation(new_client_call_details, request)
                    await call
                except grpc.RpcError:
                    pass

                self.calls.append(call)

                new_client_call_details = aio.ClientCallDetails(
                    method=client_call_details.method,
                    timeout=None,
                    metadata=client_call_details.metadata,
                    credentials=client_call_details.credentials,
                    wait_for_ready=client_call_details.wait_for_ready,
                )

                call = await continuation(new_client_call_details, request)
                self.calls.append(call)
                return call

        interceptor = RetryInterceptor()

        async with aio.insecure_channel(
            self._server_target, interceptors=[interceptor]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCallWithSleep",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(messages_pb2.SimpleRequest())

            await call

            self.assertEqual(grpc.StatusCode.OK, await call.code())

            # Check that two calls were made, first one finishing with
            # a deadline and second one finishing ok.
            self.assertEqual(len(interceptor.calls), 2)
            self.assertEqual(
                await interceptor.calls[0].code(),
                grpc.StatusCode.DEADLINE_EXCEEDED,
            )
            self.assertEqual(
                await interceptor.calls[1].code(), grpc.StatusCode.OK
            )

    async def test_retry_with_multiple_interceptors(self):
        class RetryInterceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                # Simulate retry twice
                for _ in range(2):
                    call = await continuation(client_call_details, request)
                    result = await call
                return result

        class AnotherInterceptor(aio.UnaryUnaryClientInterceptor):
            def __init__(self):
                self.called_times = 0

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                self.called_times += 1
                call = await continuation(client_call_details, request)
                result = await call
                return result

        # Create two interceptors, the retry interceptor will call another interceptor.
        retry_interceptor = RetryInterceptor()
        another_interceptor = AnotherInterceptor()
        async with aio.insecure_channel(
            self._server_target,
            interceptors=[retry_interceptor, another_interceptor],
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCallWithSleep",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(messages_pb2.SimpleRequest())

            await call

            self.assertEqual(grpc.StatusCode.OK, await call.code())
            self.assertEqual(another_interceptor.called_times, 2)

    async def test_rpcresponse(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            """Raw responses are seen as reegular calls"""

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                response = await call
                return call

        class ResponseInterceptor(aio.UnaryUnaryClientInterceptor):
            """Return a raw response"""

            response = messages_pb2.SimpleResponse()

            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                return ResponseInterceptor.response

        interceptor, interceptor_response = Interceptor(), ResponseInterceptor()

        async with aio.insecure_channel(
            self._server_target,
            interceptors=[interceptor, interceptor_response],
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(messages_pb2.SimpleRequest())
            response = await call

            # Check that the response returned is the one returned by the
            # interceptor
            self.assertEqual(id(response), id(ResponseInterceptor.response))

            # Check all of the UnaryUnaryCallResponse attributes
            self.assertTrue(call.done())
            self.assertFalse(call.cancel())
            self.assertFalse(call.cancelled())
            self.assertEqual(await call.code(), grpc.StatusCode.OK)
            self.assertEqual(await call.details(), "")
            self.assertEqual(await call.initial_metadata(), None)
            self.assertEqual(await call.trailing_metadata(), None)
            self.assertEqual(await call.debug_error_string(), None)


class TestInterceptedUnaryUnaryCall(AioTestBase):
    async def setUp(self):
        self._server_target, self._server = await start_test_server()

    async def tearDown(self):
        await self._server.stop(None)

    async def test_call_ok(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            response = await call

            self.assertTrue(call.done())
            self.assertFalse(call.cancelled())
            self.assertEqual(type(response), messages_pb2.SimpleResponse)
            self.assertEqual(await call.code(), grpc.StatusCode.OK)
            self.assertEqual(await call.details(), "")
            self.assertEqual(await call.initial_metadata(), aio.Metadata())
            self.assertEqual(await call.trailing_metadata(), aio.Metadata())

    async def test_call_ok_awaited(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                await call
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            response = await call

            self.assertTrue(call.done())
            self.assertFalse(call.cancelled())
            self.assertEqual(type(response), messages_pb2.SimpleResponse)
            self.assertEqual(await call.code(), grpc.StatusCode.OK)
            self.assertEqual(await call.details(), "")
            self.assertEqual(await call.initial_metadata(), aio.Metadata())
            self.assertEqual(await call.trailing_metadata(), aio.Metadata())

    async def test_call_rpc_error(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCallWithSleep",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(
                messages_pb2.SimpleRequest(),
                timeout=_constants.UNARY_CALL_WITH_SLEEP_VALUE / 2,
            )

            with self.assertRaises(aio.AioRpcError) as exception_context:
                await call

            self.assertTrue(call.done())
            self.assertFalse(call.cancelled())
            self.assertEqual(
                await call.code(), grpc.StatusCode.DEADLINE_EXCEEDED
            )
            self.assertEqual(await call.details(), "Deadline Exceeded")
            self.assertEqual(await call.initial_metadata(), aio.Metadata())
            self.assertEqual(await call.trailing_metadata(), aio.Metadata())

    async def test_call_rpc_error_awaited(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                await call
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCallWithSleep",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )

            call = multicallable(
                messages_pb2.SimpleRequest(),
                timeout=_constants.UNARY_CALL_WITH_SLEEP_VALUE / 2,
            )

            with self.assertRaises(aio.AioRpcError) as exception_context:
                await call

            self.assertTrue(call.done())
            self.assertFalse(call.cancelled())
            self.assertEqual(
                await call.code(), grpc.StatusCode.DEADLINE_EXCEEDED
            )
            self.assertEqual(await call.details(), "Deadline Exceeded")
            self.assertEqual(await call.initial_metadata(), aio.Metadata())
            self.assertEqual(await call.trailing_metadata(), aio.Metadata())

    async def test_cancel_before_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                interceptor_reached.set()
                await wait_for_ever

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            self.assertFalse(call.cancelled())
            self.assertFalse(call.done())

            await interceptor_reached.wait()
            self.assertTrue(call.cancel())

            with self.assertRaises(asyncio.CancelledError):
                await call

            self.assertTrue(call.cancelled())
            self.assertTrue(call.done())
            self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
            self.assertEqual(
                await call.details(), _LOCAL_CANCEL_DETAILS_EXPECTATION
            )
            self.assertEqual(await call.initial_metadata(), None)
            self.assertEqual(await call.trailing_metadata(), None)

    async def test_cancel_after_rpc(self):
        interceptor_reached = asyncio.Event()
        wait_for_ever = self.loop.create_future()

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                await call
                interceptor_reached.set()
                await wait_for_ever

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            self.assertFalse(call.cancelled())
            self.assertFalse(call.done())

            await interceptor_reached.wait()
            self.assertTrue(call.cancel())

            with self.assertRaises(asyncio.CancelledError):
                await call

            self.assertTrue(call.cancelled())
            self.assertTrue(call.done())
            self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
            self.assertEqual(
                await call.details(), _LOCAL_CANCEL_DETAILS_EXPECTATION
            )
            self.assertEqual(await call.initial_metadata(), None)
            self.assertEqual(await call.trailing_metadata(), None)

    async def test_cancel_inside_interceptor_after_rpc_awaiting(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                call.cancel()
                await call
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            with self.assertRaises(asyncio.CancelledError):
                await call

            self.assertTrue(call.cancelled())
            self.assertTrue(call.done())
            self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
            self.assertEqual(
                await call.details(), _LOCAL_CANCEL_DETAILS_EXPECTATION
            )
            self.assertEqual(await call.initial_metadata(), None)
            self.assertEqual(await call.trailing_metadata(), None)

    async def test_cancel_inside_interceptor_after_rpc_not_awaiting(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                call.cancel()
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            with self.assertRaises(asyncio.CancelledError):
                await call

            self.assertTrue(call.cancelled())
            self.assertTrue(call.done())
            self.assertEqual(await call.code(), grpc.StatusCode.CANCELLED)
            self.assertEqual(
                await call.details(), _LOCAL_CANCEL_DETAILS_EXPECTATION
            )
            self.assertEqual(await call.initial_metadata(), aio.Metadata())
            self.assertEqual(
                await call.trailing_metadata(),
                aio.Metadata(),
                "When the raw response is None, empty metadata is returned",
            )

    async def test_initial_metadata_modification(self):
        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                new_metadata = aio.Metadata(
                    *client_call_details.metadata, *_INITIAL_METADATA_TO_INJECT
                )
                new_details = aio.ClientCallDetails(
                    method=client_call_details.method,
                    timeout=client_call_details.timeout,
                    metadata=new_metadata,
                    credentials=client_call_details.credentials,
                    wait_for_ready=client_call_details.wait_for_ready,
                )
                return await continuation(new_details, request)

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            stub = test_pb2_grpc.TestServiceStub(channel)
            call = stub.UnaryCall(messages_pb2.SimpleRequest())

            # Expected to see the echoed initial metadata
            self.assertTrue(
                _common.seen_metadatum(
                    expected_key=_INITIAL_METADATA_KEY,
                    expected_value=_INITIAL_METADATA_TO_INJECT[
                        _INITIAL_METADATA_KEY
                    ],
                    actual=await call.initial_metadata(),
                )
            )
            # Expected to see the echoed trailing metadata
            self.assertTrue(
                _common.seen_metadatum(
                    expected_key=_TRAILING_METADATA_KEY,
                    expected_value=_INITIAL_METADATA_TO_INJECT[
                        _TRAILING_METADATA_KEY
                    ],
                    actual=await call.trailing_metadata(),
                )
            )
            self.assertEqual(await call.code(), grpc.StatusCode.OK)

    async def test_add_done_callback_before_finishes(self):
        called = asyncio.Event()
        interceptor_can_continue = asyncio.Event()

        def callback(call):
            called.set()

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                await interceptor_can_continue.wait()
                call = await continuation(client_call_details, request)
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())
            call.add_done_callback(callback)
            interceptor_can_continue.set()
            await call

            try:
                await asyncio.wait_for(
                    called.wait(), timeout=_TIMEOUT_CHECK_IF_CALLBACK_WAS_CALLED
                )
            except:
                self.fail("Callback was not called")

    async def test_add_done_callback_after_finishes(self):
        called = asyncio.Event()

        def callback(call):
            called.set()

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            await call

            call.add_done_callback(callback)

            try:
                await asyncio.wait_for(
                    called.wait(), timeout=_TIMEOUT_CHECK_IF_CALLBACK_WAS_CALLED
                )
            except:
                self.fail("Callback was not called")

    async def test_add_done_callback_after_finishes_before_await(self):
        called = asyncio.Event()

        def callback(call):
            called.set()

        class Interceptor(aio.UnaryUnaryClientInterceptor):
            async def intercept_unary_unary(
                self, continuation, client_call_details, request
            ):
                call = await continuation(client_call_details, request)
                return call

        async with aio.insecure_channel(
            self._server_target, interceptors=[Interceptor()]
        ) as channel:
            multicallable = channel.unary_unary(
                "/grpc.testing.TestService/UnaryCall",
                request_serializer=messages_pb2.SimpleRequest.SerializeToString,
                response_deserializer=messages_pb2.SimpleResponse.FromString,
            )
            call = multicallable(messages_pb2.SimpleRequest())

            call.add_done_callback(callback)

            await call

            try:
                await asyncio.wait_for(
                    called.wait(), timeout=_TIMEOUT_CHECK_IF_CALLBACK_WAS_CALLED
                )
            except:
                self.fail("Callback was not called")


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    unittest.main(verbosity=2)



# Copyright 2025 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import importlib.util
import os
import pathlib
import pkgutil
import sys
from typing import Optional, Sequence
import unittest

from typeguard import install_import_hook


# AIO
install_import_hook('grpc.aio')
install_import_hook('grpc.aio._channel')
install_import_hook('grpc.aio._server')
install_import_hook('grpc.aio._utils')
install_import_hook('grpc.aio._interceptor')
install_import_hook('grpc.aio._base_channel')
install_import_hook('grpc.aio._base_server')
install_import_hook('grpc.aio._typing')
install_import_hook('grpc.aio._call')
install_import_hook('grpc.aio._metadata')

# SYNC
install_import_hook('grpc')
install_import_hook('grpc._auth')
install_import_hook('grpc._channel')
install_import_hook('grpc._common')
install_import_hook('grpc._compression')
install_import_hook('grpc._interceptor')
install_import_hook('grpc._observability')
install_import_hook('grpc._plugin_wrapping')
install_import_hook('grpc._runtime_protos')
install_import_hook('grpc._server')
install_import_hook('grpc._simple_stubs')
install_import_hook('grpc._typing')
install_import_hook('grpc._utilities')

class SingleLoader:
    def __init__(
        self, target_module: str, test_patterns: Optional[list[str]] = None
    ):
        loader = unittest.TestLoader()
        loader.testNamePatterns = test_patterns
        self.suite = unittest.TestSuite()
        suites = []

        # TODO: Refactor/cleanup as discussed in
        # https://github.com/grpc/grpc/pull/41713#discussion_r2932055174
        #
        # Look in the current working directory for the test file physically
        target_file = f"{target_module}.py"
        target_module_name = None

        cwd = pathlib.Path.cwd()
        filepath = next(cwd.rglob(target_file), None)
        if not filepath:
            raise ValueError(f"Could not find target module {target_module}")
        rel_path = filepath.relative_to(cwd)
        # Remove the file extension and replace path separators with dots.
        target_module_name = str(rel_path.with_suffix('')).replace(os.sep, '.')

        spec = importlib.util.find_spec(target_module_name)
        module = importlib.util.module_from_spec(spec)
        spec.loader.exec_module(module)
        suites.append(loader.loadTestsFromModule(module))

        assert len(suites) == 1, f"Expected only 1 test module. Found {suites}"
        self.suite.addTest(suites[0])


def _convert_select_pattern(pattern):
    # Same as https://github.com/python/cpython/blob/v3.13.7/Lib/unittest/main.py#L50
    if not "*" in pattern:
        pattern = "*%s*" % pattern
    return pattern


def _arg_parser() -> argparse.ArgumentParser:
    # https://github.com/python/cpython/blob/v3.13.7/Lib/unittest/main.py#L161
    parser = argparse.ArgumentParser()
    parser.add_argument('-v', '--verbose', dest='verbosity',
                        action='store_const', const=2,
                        help='Verbose output')
    parser.add_argument('-q', '--quiet', dest='verbosity',
                        action='store_const', const=0,
                        help='Quiet output')
    parser.add_argument('--locals', dest='tb_locals',
                        action='store_true',
                        help='Show local variables in tracebacks')
    if sys.version_info >= (3, 12):
        parser.add_argument('--durations', dest='durations', type=int,
                            default=None, metavar="N",
                            help='Show the N slowest test cases (N=0 for all)')
    parser.add_argument('-f', '--failfast', dest='failfast',
                        action='store_true',
                        help='Stop on first fail or error')
    parser.add_argument('-k', dest='testNamePatterns',
                        action='append', type=_convert_select_pattern,
                        help='Only run tests which match the given substring')
    return parser


def main():
    if len(sys.argv) < 2:
        print(f"USAGE: {sys.argv[0]} TARGET_MODULE [TEST_ARGS]", file=sys.stderr)
        sys.exit(1)

    # Remove the current wrapper script from the args.
    sys.argv = sys.argv[1:]

    # Target module first.
    target_module = sys.argv[0]

    # Optional test args the rest.
    parsed_args = _arg_parser().parse_args(sys.argv[1:])
    test_kwargs = vars(parsed_args)

    if test_kwargs["verbosity"] is None:
        test_kwargs["verbosity"] = 2

    test_patterns = test_kwargs.pop("testNamePatterns")

    loader = SingleLoader(target_module, test_patterns=test_patterns)
    runner = unittest.TextTestRunner(**test_kwargs)

    result = runner.run(loader.suite)

    if not result.wasSuccessful():
        sys.exit("Test failure")


if __name__ == "__main__":
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/access_loggers/fluentd/v3/fluentd.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/access_loggers/fluentd/v3/fluentd.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.core.v3 import backoff_pb2 as envoy_dot_config_dot_core_dot_v3_dot_backoff__pb2
from envoy.config.core.v3 import extension_pb2 as envoy_dot_config_dot_core_dot_v3_dot_extension__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import struct_pb2 as google_dot_protobuf_dot_struct__pb2
from google.protobuf import wrappers_pb2 as google_dot_protobuf_dot_wrappers__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n8envoy/extensions/access_loggers/fluentd/v3/fluentd.proto\x12*envoy.extensions.access_loggers.fluentd.v3\x1a\"envoy/config/core/v3/backoff.proto\x1a$envoy/config/core/v3/extension.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1cgoogle/protobuf/struct.proto\x1a\x1egoogle/protobuf/wrappers.proto\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xcb\x04\n\x16\x46luentdAccessLogConfig\x12\x18\n\x07\x63luster\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x14\n\x03tag\x18\x02 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x1c\n\x0bstat_prefix\x18\x03 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x12\x42\n\x15\x62uffer_flush_interval\x18\x04 \x01(\x0b\x32\x19.google.protobuf.DurationB\x08\xfa\x42\x05\xaa\x01\x02*\x00\x12\x37\n\x11\x62uffer_size_bytes\x18\x05 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12\x31\n\x06record\x18\x06 \x01(\x0b\x32\x17.google.protobuf.StructB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x12\x66\n\rretry_options\x18\x07 \x01(\x0b\x32O.envoy.extensions.access_loggers.fluentd.v3.FluentdAccessLogConfig.RetryOptions\x12>\n\nformatters\x18\x08 \x03(\x0b\x32*.envoy.config.core.v3.TypedExtensionConfig\x1a\x8a\x01\n\x0cRetryOptions\x12:\n\x14max_connect_attempts\x18\x01 \x01(\x0b\x32\x1c.google.protobuf.UInt32Value\x12>\n\x0f\x62\x61\x63koff_options\x18\x02 \x01(\x0b\x32%.envoy.config.core.v3.BackoffStrategyB\xaf\x01\n8io.envoyproxy.envoy.extensions.access_loggers.fluentd.v3B\x0c\x46luentdProtoP\x01Z[github.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/fluentd/v3;fluentdv3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.access_loggers.fluentd.v3.fluentd_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n8io.envoyproxy.envoy.extensions.access_loggers.fluentd.v3B\014FluentdProtoP\001Z[github.com/envoyproxy/go-control-plane/envoy/extensions/access_loggers/fluentd/v3;fluentdv3\272\200\310\321\006\002\020\002'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['cluster']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['cluster']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['tag']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['tag']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['stat_prefix']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['stat_prefix']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['buffer_flush_interval']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['buffer_flush_interval']._serialized_options = b'\372B\005\252\001\002*\000'
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['record']._loaded_options = None
  _globals['_FLUENTDACCESSLOGCONFIG'].fields_by_name['record']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_FLUENTDACCESSLOGCONFIG']._serialized_start=329
  _globals['_FLUENTDACCESSLOGCONFIG']._serialized_end=916
  _globals['_FLUENTDACCESSLOGCONFIG_RETRYOPTIONS']._serialized_start=778
  _globals['_FLUENTDACCESSLOGCONFIG_RETRYOPTIONS']._serialized_end=916
# @@protoc_insertion_point(module_scope)



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/config/filter/http/rbac/v2/rbac.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/config/filter/http/rbac/v2/rbac.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from envoy.config.rbac.v2 import rbac_pb2 as envoy_dot_config_dot_rbac_dot_v2_dot_rbac__pb2
from udpa.annotations import migrate_pb2 as udpa_dot_annotations_dot_migrate__pb2
from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n+envoy/config/filter/http/rbac/v2/rbac.proto\x12 envoy.config.filter.http.rbac.v2\x1a\x1f\x65nvoy/config/rbac/v2/rbac.proto\x1a\x1eudpa/annotations/migrate.proto\x1a\x1dudpa/annotations/status.proto\"c\n\x04RBAC\x12)\n\x05rules\x18\x01 \x01(\x0b\x32\x1a.envoy.config.rbac.v2.RBAC\x12\x30\n\x0cshadow_rules\x18\x02 \x01(\x0b\x32\x1a.envoy.config.rbac.v2.RBAC\"J\n\x0cRBACPerRoute\x12\x34\n\x04rbac\x18\x02 \x01(\x0b\x32&.envoy.config.filter.http.rbac.v2.RBACJ\x04\x08\x01\x10\x02\x42\xc2\x01\n.io.envoyproxy.envoy.config.filter.http.rbac.v2B\tRbacProtoP\x01ZNgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/rbac/v2;rbacv2\xf2\x98\xfe\x8f\x05\'\x12%envoy.extensions.filters.http.rbac.v3\xba\x80\xc8\xd1\x06\x02\x10\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.config.filter.http.rbac.v2.rbac_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n.io.envoyproxy.envoy.config.filter.http.rbac.v2B\tRbacProtoP\001ZNgithub.com/envoyproxy/go-control-plane/envoy/config/filter/http/rbac/v2;rbacv2\362\230\376\217\005\'\022%envoy.extensions.filters.http.rbac.v3\272\200\310\321\006\002\020\001'
  _globals['_RBAC']._serialized_start=177
  _globals['_RBAC']._serialized_end=276
  _globals['_RBACPERROUTE']._serialized_start=278
  _globals['_RBACPERROUTE']._serialized_end=352
# @@protoc_insertion_point(module_scope)



# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""QPS Scenarios to run"""

_SCENARIOS = {
    "large-message-throughput": (
        '{"scenarios":[{"name":"large-message-throughput",'
        ' "spawn_local_worker_count": -2, "warmup_seconds": 30,'
        ' "benchmark_seconds": 270, "num_servers": 1, "server_config":'
        ' {"async_server_threads": 1, "security_params": null, "server_type":'
        ' "ASYNC_SERVER"}, "num_clients": 1, "client_config": {"client_type":'
        ' "ASYNC_CLIENT", "security_params": null, "payload_config":'
        ' {"simple_params": {"resp_size": 1048576, "req_size": 1048576}},'
        ' "client_channels": 1, "async_client_threads": 1,'
        ' "outstanding_rpcs_per_channel": 1, "rpc_type": "UNARY",'
        ' "load_params": {"closed_loop": {}}, "histogram_params":'
        ' {"max_possible": 60000000000.0, "resolution": 0.01}}}]}'
    ),
    "multi-channel-64-KiB": (
        '{"scenarios":[{"name":"multi-channel-64-KiB",'
        ' "spawn_local_worker_count": -3, "warmup_seconds": 30,'
        ' "benchmark_seconds": 270, "num_servers": 1, "server_config":'
        ' {"async_server_threads": 31, "security_params": null, "server_type":'
        ' "ASYNC_SERVER"}, "num_clients": 2, "client_config": {"client_type":'
        ' "ASYNC_CLIENT", "security_params": null, "payload_config":'
        ' {"simple_params": {"resp_size": 65536, "req_size": 65536}},'
        ' "client_channels": 32, "async_client_threads": 31,'
        ' "outstanding_rpcs_per_channel": 100, "rpc_type": "UNARY",'
        ' "load_params": {"closed_loop": {}}, "histogram_params":'
        ' {"max_possible": 60000000000.0, "resolution": 0.01}}}]}'
    ),
}



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# NO CHECKED-IN PROTOBUF GENCODE
# source: envoy/extensions/http/stateful_session/envelope/v3/envelope.proto
# Protobuf Python Version: 6.31.1
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import runtime_version as _runtime_version
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
_runtime_version.ValidateProtobufRuntimeVersion(
    _runtime_version.Domain.PUBLIC,
    6,
    31,
    1,
    '',
    'envoy/extensions/http/stateful_session/envelope/v3/envelope.proto'
)
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from udpa.annotations import status_pb2 as udpa_dot_annotations_dot_status__pb2
from validate import validate_pb2 as validate_dot_validate__pb2


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\nAenvoy/extensions/http/stateful_session/envelope/v3/envelope.proto\x12\x32\x65nvoy.extensions.http.stateful_session.envelope.v3\x1a\x1dudpa/annotations/status.proto\x1a\x17validate/validate.proto\"\xa2\x01\n\x14\x45nvelopeSessionState\x12i\n\x06header\x18\x01 \x01(\x0b\x32O.envoy.extensions.http.stateful_session.envelope.v3.EnvelopeSessionState.HeaderB\x08\xfa\x42\x05\x8a\x01\x02\x10\x01\x1a\x1f\n\x06Header\x12\x15\n\x04name\x18\x01 \x01(\tB\x07\xfa\x42\x04r\x02\x10\x01\x42\xc1\x01\n@io.envoyproxy.envoy.extensions.http.stateful_session.envelope.v3B\rEnvelopeProtoP\x01Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/stateful_session/envelope/v3;envelopev3\xba\x80\xc8\xd1\x06\x02\x10\x02\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'envoy.extensions.http.stateful_session.envelope.v3.envelope_pb2', _globals)
if not _descriptor._USE_C_DESCRIPTORS:
  _globals['DESCRIPTOR']._loaded_options = None
  _globals['DESCRIPTOR']._serialized_options = b'\n@io.envoyproxy.envoy.extensions.http.stateful_session.envelope.v3B\rEnvelopeProtoP\001Zdgithub.com/envoyproxy/go-control-plane/envoy/extensions/http/stateful_session/envelope/v3;envelopev3\272\200\310\321\006\002\020\002'
  _globals['_ENVELOPESESSIONSTATE_HEADER'].fields_by_name['name']._loaded_options = None
  _globals['_ENVELOPESESSIONSTATE_HEADER'].fields_by_name['name']._serialized_options = b'\372B\004r\002\020\001'
  _globals['_ENVELOPESESSIONSTATE'].fields_by_name['header']._loaded_options = None
  _globals['_ENVELOPESESSIONSTATE'].fields_by_name['header']._serialized_options = b'\372B\005\212\001\002\020\001'
  _globals['_ENVELOPESESSIONSTATE']._serialized_start=178
  _globals['_ENVELOPESESSIONSTATE']._serialized_end=340
  _globals['_ENVELOPESESSIONSTATE_HEADER']._serialized_start=309
  _globals['_ENVELOPESESSIONSTATE_HEADER']._serialized_end=340
# @@protoc_insertion_point(module_scope)



# Copyright 2021 The gRPC Authors
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import gevent
from gevent import monkey

monkey.patch_all()
threadpool = gevent.hub.get_hub().threadpool

# Currently, each channel corresponds to a single native thread in the
# gevent threadpool. Thus, when the unit test suite spins up hundreds of
# channels concurrently, some will be starved out, causing the test to
# increase in duration. We increase the max size here so this does not
# happen.
threadpool.maxsize = 1024
threadpool.size = 32

import traceback, signal
from typing import Sequence


import grpc.experimental.gevent
grpc.experimental.gevent.init_gevent()

import gevent
import greenlet
import datetime

import grpc
import unittest
import sys
import os
import pkgutil
import importlib

def trace_callback(event, args):
    if event in ("switch", "throw"):
        origin, target = args
        sys.stderr.write("{} Transfer from {} to {} with {}\n".format(datetime.datetime.now(), origin, target, event))
    else:
        sys.stderr.write("Unknown event {}.\n".format(event))
    sys.stderr.flush()

if os.getenv("GREENLET_TRACE") is not None:
    greenlet.settrace(trace_callback)

def debug(sig, frame):
    d={'_frame':frame}
    d.update(frame.f_globals)
    d.update(frame.f_locals)

    sys.stderr.write("Traceback:\n{}".format("\n".join(traceback.format_stack(frame))))
    import gevent.util; gevent.util.print_run_info()
    sys.stderr.flush()

signal.signal(signal.SIGTERM, debug)


class SingleLoader:
    def __init__(self, pattern: str):
        loader = unittest.TestLoader()
        self.suite = unittest.TestSuite()
        tests = []
        for importer, module_name, is_package in pkgutil.walk_packages([os.path.dirname(os.path.relpath(__file__))]):
            if pattern in module_name:
                spec = importer.find_spec(module_name)
                module = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(module)
                tests.append(loader.loadTestsFromModule(module))
        if len(tests) != 1:
            raise AssertionError("Expected only 1 test module. Found {}".format(tests))
        self.suite.addTest(tests[0])


    def loadTestsFromNames(self, names: Sequence[str], module: str = None) -> unittest.TestSuite:
        return self.suite

if __name__ == "__main__":

    if len(sys.argv) != 2:
        print(f"USAGE: {sys.argv[0]} TARGET_MODULE", file=sys.stderr)

    target_module = sys.argv[1]

    loader = SingleLoader(target_module)
    runner = unittest.TextTestRunner()

    result = gevent.spawn(runner.run, loader.suite)
    result.join()
    if not result.value.wasSuccessful():
        sys.exit("Test failure.")



# Copyright 2023 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import collections
from typing import List

from grpc_observability._cyobservability import MetricsName


class Metric(
    collections.namedtuple(
        "Metric",
        ["name", "cyname", "unit", "description"],
    )
):
    pass


CLIENT_ATTEMPT_STARTED = Metric(
    "grpc.client.attempt.started",
    MetricsName.CLIENT_STARTED_RPCS,
    "{attempt}",
    "Number of client call attempts started",
)
CLIENT_ATTEMPT_DURATION = Metric(
    "grpc.client.attempt.duration",
    MetricsName.CLIENT_ROUNDTRIP_LATENCY,
    "s",
    "End-to-end time taken to complete a client call attempt",
)
CLIENT_RPC_DURATION = Metric(
    "grpc.client.call.duration",
    MetricsName.CLIENT_API_LATENCY,
    "s",
    "End-to-end time taken to complete a call from client's perspective",
)
CLIENT_ATTEMPT_SEND_BYTES = Metric(
    "grpc.client.attempt.sent_total_compressed_message_size",
    MetricsName.CLIENT_SEND_BYTES_PER_RPC,
    "By",
    "Compressed message bytes sent per client call attempt",
)
CLIENT_ATTEMPT_RECEIVED_BYTES = Metric(
    "grpc.client.attempt.rcvd_total_compressed_message_size",
    MetricsName.CLIENT_RECEIVED_BYTES_PER_RPC,
    "By",
    "Compressed message bytes received per call attempt",
)
SERVER_STARTED_RPCS = Metric(
    "grpc.server.call.started",
    MetricsName.SERVER_STARTED_RPCS,
    "{call}",
    "Number of server calls started",
)
SERVER_RPC_DURATION = Metric(
    "grpc.server.call.duration",
    MetricsName.SERVER_SERVER_LATENCY,
    "s",
    "End-to-end time taken to complete a call from server transport's perspective",
)
SERVER_RPC_SEND_BYTES = Metric(
    "grpc.server.call.sent_total_compressed_message_size",
    MetricsName.SERVER_SENT_BYTES_PER_RPC,
    "By",
    "Compressed message bytes sent per server call",
)
SERVER_RPC_RECEIVED_BYTES = Metric(
    "grpc.server.call.rcvd_total_compressed_message_size",
    MetricsName.SERVER_RECEIVED_BYTES_PER_RPC,
    "By",
    "Compressed message bytes received per server call",
)


def base_metrics() -> List[Metric]:
    return [
        CLIENT_ATTEMPT_STARTED,
        CLIENT_ATTEMPT_DURATION,
        # CLIENT_RPC_DURATION is not required yet
        # CLIENT_RPC_DURATION,
        CLIENT_ATTEMPT_SEND_BYTES,
        CLIENT_ATTEMPT_RECEIVED_BYTES,
        SERVER_STARTED_RPCS,
        SERVER_RPC_DURATION,
        SERVER_RPC_SEND_BYTES,
        SERVER_RPC_RECEIVED_BYTES,
    ]



# Copyright 2015 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import argparse
import glob
import multiprocessing
import os
import pickle
import shutil
import sys
import tempfile
from typing import Dict, List, Union

import _utils
import yaml

PROJECT_ROOT = os.path.join(
    os.path.dirname(os.path.abspath(__file__)), "..", ".."
)
os.chdir(PROJECT_ROOT)
# TODO(lidiz) find a better way for plugins to reference each other
sys.path.append(os.path.join(PROJECT_ROOT, "tools", "buildgen", "plugins"))

# from tools.run_tests.python_utils import jobset
jobset = _utils.import_python_module(
    os.path.join(
        PROJECT_ROOT, "tools", "run_tests", "python_utils", "jobset.py"
    )
)

PREPROCESSED_BUILD = ".preprocessed_build"
test = {} if os.environ.get("TEST", "false") == "true" else None

assert sys.argv[1:], "run generate_projects.sh instead of this directly"
parser = argparse.ArgumentParser()
parser.add_argument(
    "build_files",
    nargs="+",
    default=[],
    help="build files describing build specs",
)
parser.add_argument(
    "--templates", nargs="+", default=[], help="mako template files to render"
)
parser.add_argument(
    "--output_merged",
    "-m",
    default="",
    type=str,
    help="merge intermediate results to a file",
)
parser.add_argument(
    "--jobs",
    "-j",
    default=multiprocessing.cpu_count(),
    type=int,
    help="maximum parallel jobs",
)
parser.add_argument(
    "--base", default=".", type=str, help="base path for generated files"
)
args = parser.parse_args()


def preprocess_build_files() -> _utils.Bunch:
    """Merges build yaml into a one dictionary then pass it to plugins."""
    build_spec = dict()
    for build_file in args.build_files:
        with open(build_file, "r") as f:
            _utils.merge_json(build_spec, yaml.safe_load(f.read()))
    # Executes plugins. Plugins update the build spec in-place.
    for py_file in sorted(glob.glob("tools/buildgen/plugins/*.py")):
        plugin = _utils.import_python_module(py_file)
        plugin.mako_plugin(build_spec)
    if args.output_merged:
        with open(args.output_merged, "w") as f:
            f.write(yaml.dump(build_spec))
    # Makes build_spec sort of immutable and dot-accessible
    return _utils.to_bunch(build_spec)


def generate_template_render_jobs(templates: List[str]) -> List[jobset.JobSpec]:
    """Generate JobSpecs for each one of the template rendering work."""
    jobs = []
    base_cmd = [sys.executable, "tools/buildgen/_mako_renderer.py"]
    for template in sorted(templates, reverse=True):
        root, f = os.path.split(template)
        if os.path.splitext(f)[1] == ".template":
            out_dir = args.base + root[len("templates") :]
            out = os.path.join(out_dir, os.path.splitext(f)[0])
            if not os.path.exists(out_dir):
                os.makedirs(out_dir)
            cmd = base_cmd[:]
            cmd.append("-P")
            cmd.append(PREPROCESSED_BUILD)
            cmd.append("-o")
            if test is None:
                cmd.append(out)
            else:
                tf = tempfile.mkstemp()
                test[out] = tf[1]
                os.close(tf[0])
                cmd.append(test[out])
            cmd.append(args.base + "/" + root + "/" + f)
            jobs.append(
                jobset.JobSpec(cmd, shortname=out, timeout_seconds=None)
            )
    return jobs


def main() -> None:
    templates = args.templates
    if not templates:
        for root, _, files in os.walk("templates"):
            for f in files:
                templates.append(os.path.join(root, f))

    build_spec = preprocess_build_files()
    with open(PREPROCESSED_BUILD, "wb") as f:
        pickle.dump(build_spec, f)

    err_cnt, _ = jobset.run(
        generate_template_render_jobs(templates), maxjobs=args.jobs
    )
    if err_cnt != 0:
        print(
            "ERROR: %s error(s) found while generating projects." % err_cnt,
            file=sys.stderr,
        )
        sys.exit(1)

    if test is not None:
        for s, g in test.items():
            if os.path.isfile(g):
                assert 0 == os.system("diff %s %s" % (s, g)), s
                os.unlink(g)
            else:
                assert 0 == os.system("diff -r %s %s" % (s, g)), s
                shutil.rmtree(g, ignore_errors=True)


if __name__ == "__main__":
    main()



#!/usr/bin/env python3

# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

import os
import re
import sys

os.chdir(os.path.join(os.path.dirname(sys.argv[0]), "../../.."))

errors = 0
tracers = []
pattern = re.compile('GRPC_TRACER_INITIALIZER\((true|false), "(.*)"\)')
for root, dirs, files in os.walk("src/core"):
    for filename in files:
        path = os.path.join(root, filename)
        if os.path.splitext(path)[1] != ".c":
            continue
        with open(path) as f:
            text = f.read()
        for o in pattern.findall(text):
            tracers.append(o[1])

with open("doc/environment_variables.md") as f:
    text = f.read()

for t in tracers:
    if t not in text:
        print(
            'ERROR: tracer "%s" is not mentioned in'
            " doc/environment_variables.md" % t
        )
        errors += 1

assert errors == 0



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests server and client side metadata API."""

import logging
import unittest
import weakref

import grpc
from grpc import _channel

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_CHANNEL_ARGS = (
    ("grpc.primary_user_agent", "primary-agent"),
    ("grpc.secondary_user_agent", "secondary-agent"),
)

_REQUEST = b"\x00\x00\x00"
_RESPONSE = b"\x00\x00\x00"

_SERVICE_NAME = "test"
_UNARY_UNARY = "UnaryUnary"
_UNARY_STREAM = "UnaryStream"
_STREAM_UNARY = "StreamUnary"
_STREAM_STREAM = "StreamStream"

_INVOCATION_METADATA = (
    (
        b"invocation-md-key",
        "invocation-md-value",
    ),
    (
        "invocation-md-key-bin",
        b"\x00\x01",
    ),
)
_EXPECTED_INVOCATION_METADATA = (
    (
        "invocation-md-key",
        "invocation-md-value",
    ),
    (
        "invocation-md-key-bin",
        b"\x00\x01",
    ),
)

_INITIAL_METADATA = (
    (b"initial-md-key", "initial-md-value"),
    ("initial-md-key-bin", b"\x00\x02"),
)
_EXPECTED_INITIAL_METADATA = (
    (
        "initial-md-key",
        "initial-md-value",
    ),
    (
        "initial-md-key-bin",
        b"\x00\x02",
    ),
)

_TRAILING_METADATA = (
    (
        "server-trailing-md-key",
        "server-trailing-md-value",
    ),
    (
        "server-trailing-md-key-bin",
        b"\x00\x03",
    ),
)
_EXPECTED_TRAILING_METADATA = _TRAILING_METADATA


def _user_agent(metadata):
    for key, val in metadata:
        if key == "user-agent":
            return val
    raise KeyError("No user agent!")


def validate_client_metadata(test, servicer_context):
    invocation_metadata = servicer_context.invocation_metadata()
    test.assertTrue(
        test_common.metadata_transmitted(
            _EXPECTED_INVOCATION_METADATA, invocation_metadata
        )
    )
    user_agent = _user_agent(invocation_metadata)
    test.assertTrue(
        user_agent.startswith("primary-agent " + _channel._USER_AGENT)
    )
    test.assertTrue(user_agent.endswith("secondary-agent"))


def handle_unary_unary(test, request, servicer_context):
    validate_client_metadata(test, servicer_context)
    servicer_context.send_initial_metadata(_INITIAL_METADATA)
    servicer_context.set_trailing_metadata(_TRAILING_METADATA)
    return _RESPONSE


def handle_unary_stream(test, request, servicer_context):
    validate_client_metadata(test, servicer_context)
    servicer_context.send_initial_metadata(_INITIAL_METADATA)
    servicer_context.set_trailing_metadata(_TRAILING_METADATA)
    for _ in range(test_constants.STREAM_LENGTH):
        yield _RESPONSE


def handle_stream_unary(test, request_iterator, servicer_context):
    validate_client_metadata(test, servicer_context)
    servicer_context.send_initial_metadata(_INITIAL_METADATA)
    servicer_context.set_trailing_metadata(_TRAILING_METADATA)
    # TODO(issue:#6891) We should be able to remove this loop
    for request in request_iterator:
        pass
    return _RESPONSE


def handle_stream_stream(test, request_iterator, servicer_context):
    validate_client_metadata(test, servicer_context)
    servicer_context.send_initial_metadata(_INITIAL_METADATA)
    servicer_context.set_trailing_metadata(_TRAILING_METADATA)
    # TODO(issue:#6891) We should be able to remove this loop,
    # and replace with return; yield
    for request in request_iterator:
        yield _RESPONSE


class _MethodHandler(grpc.RpcMethodHandler):
    def __init__(self, test, request_streaming, response_streaming):
        self.request_streaming = request_streaming
        self.response_streaming = response_streaming
        self.request_deserializer = None
        self.response_serializer = None
        self.unary_unary = None
        self.unary_stream = None
        self.stream_unary = None
        self.stream_stream = None
        if self.request_streaming and self.response_streaming:
            self.stream_stream = lambda x, y: handle_stream_stream(test, x, y)
        elif self.request_streaming:
            self.stream_unary = lambda x, y: handle_stream_unary(test, x, y)
        elif self.response_streaming:
            self.unary_stream = lambda x, y: handle_unary_stream(test, x, y)
        else:
            self.unary_unary = lambda x, y: handle_unary_unary(test, x, y)


def get_method_handlers(test):
    return {
        _UNARY_UNARY: _MethodHandler(test, False, False),
        _UNARY_STREAM: _MethodHandler(test, False, True),
        _STREAM_UNARY: _MethodHandler(test, True, False),
        _STREAM_STREAM: _MethodHandler(test, True, True),
    }


class MetadataTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server()
        self._server.add_registered_method_handlers(
            _SERVICE_NAME, get_method_handlers(weakref.proxy(self))
        )
        port = self._server.add_insecure_port("[::]:0")
        self._server.start()
        self._channel = grpc.insecure_channel(
            "localhost:%d" % port, options=_CHANNEL_ARGS
        )

    def tearDown(self):
        self._server.stop(0)
        self._channel.close()

    def testUnaryUnary(self):
        multi_callable = self._channel.unary_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_UNARY),
            _registered_method=True,
        )
        unused_response, call = multi_callable.with_call(
            _REQUEST, metadata=_INVOCATION_METADATA
        )
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_INITIAL_METADATA, call.initial_metadata()
            )
        )
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_TRAILING_METADATA, call.trailing_metadata()
            )
        )

    def testUnaryStream(self):
        multi_callable = self._channel.unary_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_STREAM),
            _registered_method=True,
        )
        call = multi_callable(_REQUEST, metadata=_INVOCATION_METADATA)
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_INITIAL_METADATA, call.initial_metadata()
            )
        )
        for _ in call:
            pass
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_TRAILING_METADATA, call.trailing_metadata()
            )
        )

    def testStreamUnary(self):
        multi_callable = self._channel.stream_unary(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_UNARY),
            _registered_method=True,
        )
        unused_response, call = multi_callable.with_call(
            iter([_REQUEST] * test_constants.STREAM_LENGTH),
            metadata=_INVOCATION_METADATA,
        )
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_INITIAL_METADATA, call.initial_metadata()
            )
        )
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_TRAILING_METADATA, call.trailing_metadata()
            )
        )

    def testStreamStream(self):
        multi_callable = self._channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_STREAM),
            _registered_method=True,
        )
        call = multi_callable(
            iter([_REQUEST] * test_constants.STREAM_LENGTH),
            metadata=_INVOCATION_METADATA,
        )
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_INITIAL_METADATA, call.initial_metadata()
            )
        )
        for _ in call:
            pass
        self.assertTrue(
            test_common.metadata_transmitted(
                _EXPECTED_TRAILING_METADATA, call.trailing_metadata()
            )
        )


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2018 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""Tests server and client side compression."""

import itertools
import logging
import threading
import time
import unittest

import grpc

from tests.unit import test_common
from tests.unit.framework.common import test_constants

_BEAT = 0.5
_SOME_TIME = 5
_MORE_TIME = 10

_SERVICE_NAME = ""
_STREAM_URI = "Meffod"
_UNARY_URI = "MeffodMan"


class _StreamingMethodHandler(grpc.RpcMethodHandler):
    request_streaming = True
    response_streaming = True
    request_deserializer = None
    response_serializer = None

    def stream_stream(self, request_iterator, servicer_context):
        for request in request_iterator:
            yield request * 2


class _UnaryMethodHandler(grpc.RpcMethodHandler):
    request_streaming = False
    response_streaming = False
    request_deserializer = None
    response_serializer = None

    def unary_unary(self, request, servicer_context):
        return request * 2


_STREAMING_METHOD_HANDLER = _StreamingMethodHandler()
_UNARY_METHOD_HANDLER = _UnaryMethodHandler()


_METHOD_HANDLERS = {
    _STREAM_URI: _STREAMING_METHOD_HANDLER,
    _UNARY_URI: _UNARY_METHOD_HANDLER,
}


class _Pipe:
    def __init__(self, values):
        self._condition = threading.Condition()
        self._values = list(values)
        self._open = True

    def __iter__(self):
        return self

    def _next(self):
        with self._condition:
            while not self._values and self._open:
                self._condition.wait()
            if self._values:
                return self._values.pop(0)
            else:
                raise StopIteration()

    def next(self):
        return self._next()

    def __next__(self):
        return self._next()

    def add(self, value):
        with self._condition:
            self._values.append(value)
            self._condition.notify()

    def close(self):
        with self._condition:
            self._open = False
            self._condition.notify()

    def __enter__(self):
        return self

    def __exit__(self, type, value, traceback):
        self.close()


class ChannelCloseTest(unittest.TestCase):
    def setUp(self):
        self._server = test_common.test_server(
            max_workers=test_constants.THREAD_CONCURRENCY
        )
        self._server.add_registered_method_handlers("", _METHOD_HANDLERS)
        self._port = self._server.add_insecure_port("[::]:0")
        self._server.start()

    def tearDown(self):
        self._server.stop(None)

    def test_close_immediately_after_call_invocation(self):
        channel = grpc.insecure_channel("localhost:{}".format(self._port))
        multi_callable = channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
            _registered_method=True,
        )
        request_iterator = _Pipe(())
        response_iterator = multi_callable(request_iterator)
        channel.close()
        request_iterator.close()

        self.assertIs(response_iterator.code(), grpc.StatusCode.CANCELLED)

    def test_close_while_call_active(self):
        channel = grpc.insecure_channel("localhost:{}".format(self._port))
        multi_callable = channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
            _registered_method=True,
        )
        request_iterator = _Pipe((b"abc",))
        response_iterator = multi_callable(request_iterator)
        next(response_iterator)
        channel.close()
        request_iterator.close()

        self.assertIs(response_iterator.code(), grpc.StatusCode.CANCELLED)

    def test_context_manager_close_while_call_active(self):
        with grpc.insecure_channel(
            "localhost:{}".format(self._port)
        ) as channel:
            multi_callable = channel.stream_stream(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
                _registered_method=True,
            )
            request_iterator = _Pipe((b"abc",))
            response_iterator = multi_callable(request_iterator)
            next(response_iterator)
        request_iterator.close()

        self.assertIs(response_iterator.code(), grpc.StatusCode.CANCELLED)

    def test_context_manager_close_while_many_calls_active(self):
        with grpc.insecure_channel(
            "localhost:{}".format(self._port)
        ) as channel:
            multi_callable = channel.stream_stream(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
                _registered_method=True,
            )
            request_iterators = tuple(
                _Pipe((b"abc",))
                for _ in range(test_constants.THREAD_CONCURRENCY)
            )
            response_iterators = []
            for request_iterator in request_iterators:
                response_iterator = multi_callable(request_iterator)
                next(response_iterator)
                response_iterators.append(response_iterator)
        for request_iterator in request_iterators:
            request_iterator.close()

        for response_iterator in response_iterators:
            self.assertIs(response_iterator.code(), grpc.StatusCode.CANCELLED)

    def test_many_concurrent_closes(self):
        channel = grpc.insecure_channel("localhost:{}".format(self._port))
        multi_callable = channel.stream_stream(
            grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
            _registered_method=True,
        )
        request_iterator = _Pipe((b"abc",))
        response_iterator = multi_callable(request_iterator)
        next(response_iterator)
        start = time.time()
        end = start + _MORE_TIME

        def sleep_some_time_then_close():
            time.sleep(_SOME_TIME)
            channel.close()

        for _ in range(test_constants.THREAD_CONCURRENCY):
            close_thread = threading.Thread(target=sleep_some_time_then_close)
            close_thread.start()
        while True:
            request_iterator.add(b"def")
            time.sleep(_BEAT)
            if end < time.time():
                break
        request_iterator.close()

        self.assertIs(response_iterator.code(), grpc.StatusCode.CANCELLED)

    def test_exception_in_callback(self):
        with grpc.insecure_channel(
            "localhost:{}".format(self._port)
        ) as channel:
            stream_multi_callable = channel.stream_stream(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _STREAM_URI),
                _registered_method=True,
            )
            endless_iterator = itertools.repeat(b"abc")
            stream_response_iterator = stream_multi_callable(endless_iterator)
            future = channel.unary_unary(
                grpc._common.fully_qualified_method(_SERVICE_NAME, _UNARY_URI),
                _registered_method=True,
            ).future(b"abc")

            def on_done_callback(future):
                raise Exception("This should not cause a deadlock.")

            future.add_done_callback(on_done_callback)
            future.result()


if __name__ == "__main__":
    logging.basicConfig()
    unittest.main(verbosity=2)



# Copyright 2016 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""The entry point for the qps worker."""

import argparse
import logging
import time

import grpc

from src.proto.grpc.testing import worker_service_pb2_grpc
from tests.qps import worker_server
from tests.unit import test_common


def run_worker_server(driver_port, server_port):
    server = test_common.test_server()
    servicer = worker_server.WorkerServer(server_port)
    worker_service_pb2_grpc.add_WorkerServiceServicer_to_server(
        servicer, server
    )
    server.add_insecure_port("[::]:{}".format(driver_port))
    server.start()
    servicer.wait_for_quit()
    server.stop(0)


if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    parser = argparse.ArgumentParser(
        description="gRPC Python performance testing worker"
    )
    parser.add_argument(
        "--driver_port",
        type=int,
        dest="driver_port",
        help="The port for the worker to expose for driver communication",
    )
    parser.add_argument(
        "--server_port",
        type=int,
        default=None,
        dest="server_port",
        help=(
            "The port for the server if not specified by server config message"
        ),
    )
    args = parser.parse_args()

    run_worker_server(args.driver_port, args.server_port)



#!/usr/bin/env python3
# Copyright 2017 gRPC authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
"""
Wrapper around port server starting code.

Used by developers who wish to run individual C/C++ tests outside of the
run_tests.py infrastructure.

The path to this file is called out in test/core/test_util/port.c, and printed as
an error message to users.
"""

import python_utils.start_port_server as start_port_server

start_port_server.start_port_server()

print("Port server started successfully")
